%---------- Discretization constant ----------
T = 0.010;

%---------- System constants ----------
R = 288;
T_0 = 293;
P_0 = 100000;
P_S = 900000;
V_0 = 0.0008;
A = 0.0123;
M = 10;
D_v = 5000;

%---------- Integration constants and limits ----------
v_0 = 0;
y_0 = 0;
zeta_0 = (V_0 + y_0*A)*P_0;

y_ub = 0.025;
y_lb = -0.001;

v_ub = 0.2;
v_lb = -0.2;

omega_ub = 0.031;
omega_lb = -0.031;

zeta_ub = (V_0 + y_ub*A)*P_S;
zeta_lb = (V_0 + y_lb*A)*P_0;

%---------- Backstepping constants c_i > 0 ----------
c_1 = 1000; % For the continuous-time model
c_2 = 1000; 
c_3 = 1000; 

c_1e = 400; % For the sampled-data system with emulation control
c_2e = 400; 
c_3e = 25; 

c_1d = 400; % For the sampled-data system with direct discrete-time control
c_2d = 400; 
c_3d = 25; 

%---------- Constants from the backstepping procedure ----------
K_1 = -1 - (c_1 * c_2);
K_2 = -c_2 - (c_1 * M);
K_3 = 1 + (c_1 * c_2);
K_4 = c_2 + (c_1 * M);

K_1e = -1 - (c_1e * c_2e);
K_2e = -c_2e - (c_1e * M);
K_3e = 1 + (c_1e * c_2e);
K_4e = c_2e + (c_1e * M);

K_1d = -1 - (c_1d * c_2d);
K_2d = -c_2d - (c_1d * M);
K_3d = 1 + (c_1d * c_2d);
K_4d = c_2d + (c_1d * M);

K_2_hatd = -c_2d + (c_1d * M);
K_4_hatd = c_2d - (c_1d * M);

%---------- Load characteristics ----------
theta_l = [-1.5 -0.5 4.8 6.8 6.7 6.6 6.6 6.6 6.7 7.9]'*1e3;   % [N]
c_l = [-2 0 1.5 3 4.5 6 8.5 11.5 15 18.5]'*1e-3;                % Center offsets [m]
d_l = [3.5 3 3 3 3.5 4.5 5.5 6.5 6.5 6.5]'*1e-3;                % Practical support widths [m]
w_l = 3./d_l; 
y_l = linspace(-5,25,200)*1e-3; 
[Phi_l,dPhi_l,ddPhi_l] = phi_M_gaussian3(y_l,c_l,w_l);          % Basis functions
Mphi_l = [y_l; Phi_l];                                        % Look-up table
Mdphi_l = [y_l; dPhi_l];

%---------- Reference trajectory filter ----------
lambda_w = 400;

k0 = 1 * lambda_w^4;
k1 = 4 * lambda_w^3;
k2 = 6 * lambda_w^2;
k3 = 4 * lambda_w^1;

A_z = [  0   1   0   0 ;
         0   0   1   0 ;
         0   0   0   1 ;
       -k0 -k1 -k2 -k3];
   
B_z = [0 0 0 k0]';

%---------- Simulation and plots ----------
sim('third_order_model');
sim('third_order_model_emul');
sim('third_order_model_disc');

yc_yr = y - yr;
ye_yr = ye - yre;
yd_yr = yd - yrd;


%plot(t,r,'k:');
%hold
%plot(t,yr,'k--');
%hold

plot(t,y,'k-');
hold
plot(te,ye,'g--');
plot(td,yd,'r:');
LEGEND('y_c','y_e','y_d');
TITLE('Sampling interval: T = 50ms');
XLABEL('Time (s)');
YLABEL('Position (m)');

%plot(t,yc_yr,'k-');
%hold
%plot(te,ye_yr,'g--');
%plot(td,yd_yr,'r:');
%LEGEND('y_r - y_c','y_r - y_e','y_r - y_d');
%TITLE('Sampling interval: T = 30ms');
%XLABEL('Time (s)');
%YLABEL('Error (m)');

%------------------------------------------

