%---------- Discretization constant ----------
T = 0.030;

%---------- System constants ----------
R = 288;
T_0 = 293;
P_0 = 1e5;
P_e = P_0;
P_s = 9e5;
V_0 = 0.0008;
rho_0 = 1.185;
A = 0.0123;
M = 10;

D_v	= 2.071912421740924e+003;
K_f = 4.659530303747875e+004;
D_f = 1.000000131326377e+002;
F_d = 2.099999993757047e+002;

C_v = 2.895032495811626e-008;
tau_v = 0.03833788012589;

epsilon = 1e-4;

%---------- Integration constants and limit ----------
y_0= 0;           
v_0= 0.001; 
zeta_0 = (V_0 + y_0*A)*P_0;
omega_0 = 0;
y_f_0 = 1 / K_f * F_d * sign(v_0);

     
y_lb = -0.005;
y_ub = 0.025;

zeta_ub = (A*y_ub + V_0)*P_s;
zeta_lb = (A*y_lb + V_0)*P_e;

omega_lb = -0.031;
omega_ub = 0.031;

v_ub = 0.2;
v_lb = -0.2;

y_f_ub = 5e-3;
y_f_lb = -5e-3;

%---------- Backstepping constants c_i > 0 ----------
c_1 = 800; % For the continuous-time model
c_2 = 800; %
c_3 = 30; %
c_4 = 350; %

c_1e = 600; % For the emulation model
c_2e = 600; %
c_3e = 40; %
c_4e = 300; %

c_1d = 600; % For the discrete-time model
c_2d = 600; %
c_3d = 40; %
c_4d = 300; %

%---------- Constants from the backstepping procedure ----------
K_1 = -1 - (c_1 * c_2);
K_2 = -c_2 - (c_1 * M);
K_3 = 1 + (c_1 * c_2);
K_4 = c_2 + (c_1 * M);
K_5 = -c_1 + (c_3 * K_1);
K_6 = -1 + (c_3*K_2) + K_1;
K_7 = c_1 + (c_3*K_3);
K_8 = 1 + (c_3*K_4) + K_3;
K_9 = (c_3*M) + K_4;

K_1e = -1 - (c_1e * c_2e);
K_2e = -c_2e - (c_1e * M);
K_3e = 1 + (c_1e * c_2e);
K_4e = c_2e + (c_1e * M);
K_5e = -c_1e + (c_3e * K_1e);
K_6e = -1 + (c_3e*K_2e) + K_1e;
K_7e = c_1e + (c_3e*K_3e);
K_8e = 1 + (c_3e*K_4e) + K_3e;
K_9e = (c_3e*M) + K_4e;

K_1d = -1 - (c_1d * c_2d);
K_2d = -c_2d - (c_1d * M);
K_3d = 1 + (c_1d * c_2d);
K_4d = c_2d + (c_1d * M);
K_5d = -c_1d + (c_3d * K_1d);
K_6d = -1 + (c_3d*K_2d) + K_1d;
K_7d = c_1d + (c_3d*K_3d);
K_8d = 1 + (c_3d*K_4d) + K_3d;
K_9d = (c_3d*M) + K_4d;

K_6_hatd = -1 + (c_3d*K_2d) - K_1d;
K_8_hatd = 1 + (c_3d*K_4d) - K_3d;
K_9_hatd = (c_3d*M) - K_4d;

%---------- Load characteristics ----------
theta_l = 1.0e+003 *[  -0.592094643997316   .05111147973241 2.00000000000085   2.63759736205582   3.20832821188495 3.16062161416013   2.95958874731289   2.95585608064665  2.96824124238484   2.98290116473576]';
c_l = [-2 0 1.5 3 4.5 6 8.5 11.5 15 18.5]'*1e-3;
d_l =[   0.00699951204684   0.00475291228824   0.00699972558903 0.00700017245381   0.00749955164514   0.00739759618261 0.00640862073634   0.00812905992460   0.00489113725834   0.00751633405395]';
w_l = 3./d_l; 
y_l = linspace(-5,25,200)*1e-3; 

[Phi_l,dPhi_l,ddPhi_l] = phi_M_gaussian3(y_l,c_l,w_l);         
Mphi_l = [y_l; Phi_l];                                        
Mdphi_l = [y_l; dPhi_l];
Mddphi_l = [y_l; ddPhi_l];

%---------- Reference trajectory filter ----------
lambda_w = 500;

k0 = 01 * lambda_w^5;
k1 = 05 * lambda_w^4;
k2 = 10 * lambda_w^3;
k3 = 10 * lambda_w^2;
k4 = 05 * lambda_w^1;

A_z = [  0   1   0   0   0 ;
         0   0   1   0   0 ;
         0   0   0   1   0 ;
         0   0   0   0   1 ;
       -k0 -k1 -k2 -k3 -k4];
   
B_z = [0 0 0 0 k0]';



%---------- Simulation and plots ----------
sim('fifth_order_model');
sim('fifth_order_model_emul');
sim('fifth_order_model_disc');


yc_yr = y - yr;
ye_yr = ye - yre;
yd_yr = yd - yrd;

%plot(t,r,'k:');
%hold

%plot(t,yr,'k--');
%hold
plot(t,y,'k-');
hold
plot(te,ye,'g--');
plot(td,yd,'r:');
legend('y_c','y_e','y_d');
title('Sampling interval: T = 10ms');
xlabel('Time (s)');
ylabel('Position (m)');
%------------------------------------------
%plot(t,yc_yr,'k-');
%hold
%plot(te,ye_yr,'g--');
%plot(td,yd_yr,'r:');
%LEGEND('y_c','y_e','y_d');
%TITLE('Sampling interval: T = 10ms');
%XLABEL('Time (s)');
%YLABEL('Error (m)');


